unit fEntityInspector;

interface

uses
  Windows, Messages, Forms, Grids, SysUtils, Math, Controls, StdCtrls, Classes, ComCtrls,
  CADImage, DXFConv, sgDrawingNavigator, sgConsts, ExtCtrls, Buttons, Properties,
  Dialogs, Menus, XMLInterface, ShellApi, Graphics;

type

  TsgOnSetValue = procedure of object;

  TfmEntityInspector = class(TForm)
    StatusBar: TStatusBar;
    StringGridObject: TStringGrid;
    pnlCombo: TPanel;
    SpeedButtonStayOnTop: TSpeedButton;
    cbSelectedEntities: TComboBox;
    ComboBox1: TComboBox;
    PopupMenu1: TPopupMenu;
    SavetoXML1: TMenuItem;
    SaveDialog1: TSaveDialog;
    SaveDrawing1: TMenuItem;
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StringGridObjectSelectCell(Sender: TObject; ACol,
      ARow: Integer; var CanSelect: Boolean);
    procedure StringGridObjectExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButtonStayOnTopMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure pnlComboResize(Sender: TObject);
    procedure StringGridObjectDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure StringGridObjectKeyPress(Sender: TObject; var Key: Char);
    procedure ComboBox1Change(Sender: TObject);
    procedure ComboBox1Exit(Sender: TObject);
    procedure StringGridObjectMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure cbSelectedEntitiesChange(Sender: TObject);
    procedure StringGridObjectMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SavetoXML1Click(Sender: TObject);
    procedure SaveDrawing1Click(Sender: TObject);
  private
    FDNavigator: TsgDrawingNavigator;
    FColumn, FRow: Integer;// selected cell
    FChangeCellText, FTransition: Boolean;
    FPrevCellText: string;
    FOnSetValue: TsgOnSetValue;
    FEntities: TList;
    FProperties: TList;  //list of classtype
    FMinuses: TList;
    FVertexIndex: TsgIndex;
    FEntitySample: TsgDXFEntitySample;
    FIndexesAsPluses: Boolean;
    function GetCADImage: TsgCADImage;
    procedure SetChangeCellText(const Value: Boolean);
    procedure SetOnSetValue(const Value: TsgOnSetValue);
    procedure SetIndexesAsPluses(const Value: Boolean);
    property CADImage: TsgCADImage read GetCADImage;
    property ChangeCellText: Boolean read FChangeCellText write SetChangeCellText;
  public
    procedure AddEntity(AEnt: TsgDXFEntity; ARefresh: Boolean);
    procedure Clear;
    procedure ChangeObjectParams(AProperty: TsgObject;  AEntity: TsgDXFEntity = nil);
    procedure Load(AImage: TsgDrawingNavigator);
    procedure RefreshGrid(AEntities: TList; AType: TsgObjectClass); overload;
    procedure RefreshGrid(AEntity: TsgListObject); overload;
    procedure UpdateEntity(AEntity: TsgDXFEntity);
    property OnSetValue: TsgOnSetValue read FOnSetValue write SetOnSetValue;
    property IndexesAsPluses: Boolean read FIndexesAsPluses write SetIndexesAsPluses;
  end;

var
  fmEntityInspector: TfmEntityInspector;

implementation
{$INCLUDE SGDXF.INC}

const
  ROW_NAME      = 1;
  ROW_LAYER     = 2;
  ROW_COLOR     = 3;
  ROW_BOXLEFT   = 4;
  ROW_BOXTOP    = 5;
  ROW_BOXRIGHT  = 6;
  ROW_BOXBOTTOM = 7;
  ROW_COUNT     = 8;

{$R *.dfm}

procedure TfmEntityInspector.FormCreate(Sender: TObject);
begin
  FIndexesAsPluses := False;
  FMinuses := TList.Create;
  FEntitySample := TsgDXFEntitySample.Create;
  FVertexIndex := TsgIndex.Create;
  FEntities := TList.Create;
  FProperties := TList.Create;
  FormStyle := fsStayOnTop;
  SpeedButtonStayOnTop.Down := True;
  {$IFDEF SGDEL_5}
  BiDiMode := bdLeftToRight;
  DockSite := True;
  DragKind := dkDock;
  DragMode := dmAutomatic;
  OldCreateOrder := True;
  ParentBiDiMode := False;
  UseDockManager := True;
  cbSelectedEntities.Anchors := [akLeft, akTop, akRight];
  StringGridObject.BiDiMode := bdLeftToRight;
  StringGridObject.ParentBiDiMode := False;
  {$ENDIF}
  // To forbid editing
  //StringGridObject.Options := StringGridObject.Options - [goEditing];
  FTransition := False;
  ChangeCellText := False;
  FColumn := -1;
  FRow := -1;
  StringGridObject.Width:= fmEntityInspector.ClientWidth;
  StringGridObject.Rows[0].Strings[0] := ' Property ';
  StringGridObject.Rows[0].Strings[1] := ' Value ';
  StringGridObject.DefaultRowHeight := ComboBox1.Height;
 { combobox}
  ComboBox1.Visible := False;
  StringGridObject.RowCount := 1;
end;

procedure TfmEntityInspector.FormDestroy(Sender: TObject);
begin
  // Restore inherited ComboBox WindowProc property
  FMinuses.Free;
  FEntitySample.Free;
  FVertexIndex.Free;
  FProperties.Free;
  FEntities.Free;
end;

procedure TfmEntityInspector.FormResize(Sender: TObject);
begin
  StringGridObject.DefaultColWidth := Ceil(Extended(StringGridObject.Width/2.0))-2;
end;

procedure TfmEntityInspector.RefreshGrid(AEntities: TList; AType: TsgObjectClass);
var
  I, vInd, vIndexOf, vStackCount: Integer;
  vList, vTypes, vCounts: TList;
  vObj: TsgObject;
  vListObj: TsgListObject;
  vEntity: TsgDXFEntity;
  vFirstIterate: Boolean;
  vPointClass: TsgFPointClass;

  procedure AddRow;
  begin
    if vFirstIterate then
        StringGridObject.RowCount := StringGridObject.RowCount + 1;
    Inc(vInd);
  end;

  procedure AddValue(ACaption: string; AProperty: TsgObject; AEntity: TsgObject);
  begin
    if not vFirstIterate then
    begin
      if StringGridObject.Rows[vInd].Strings[1] <>  ACaption {not TsgObject(StringGridObject.Rows[vInd].Objects[0]).IsSameValue(AObject)} then
        StringGridObject.Rows[vInd].Strings[1] := '*Varies*';
    end
    else
    begin
      StringGridObject.Rows[vInd].Strings[1] := ACaption;
      StringGridObject.Rows[vInd].Objects[0] := AProperty;
      StringGridObject.Rows[vInd].Objects[1] := AEntity;
    end;
  end;

  procedure FindSameProperties;
  var
    I, J, K: Integer;
  begin
    for I := 0 to FEntities.Count - 1 do
    begin
      if (AType <> nil) and (AType <> TObject(FEntities[I]).ClassType) then
        Continue;
      vList := TsgListObject(FEntities[I]).GetProperties;
      vIndexOf := vTypes.IndexOf(TObject(FEntities[I]).ClassType);
      if vIndexOf = -1 then
      begin
        vTypes.Add(TObject(FEntities[I]).ClassType);
        vCounts.Add(Pointer(1));
      end
      else
      begin
        vCounts[vIndexOf] := Pointer(Integer(vCounts[vIndexOf]) + 1);
        Continue; // no need to check properties for the same type
      end;
      if FProperties.Count = 0 then
        for J := 0 to vList.Count - 1 do
          FProperties.Add(TClass(TObject(vList[J]).ClassType))
      else
      begin
        for J := FProperties.Count - 1 downto 0 do
        begin
          vIndexOf := -1;
          for K := 0 to vList.Count - 1 do
            if TObject(vList[K]).ClassType =  TClass(FProperties[J]) then
              vIndexOf := K;
          if vIndexOf = -1 then
            FProperties.Delete(J);
        end;
      end;
    end;
  end;

  procedure AddProperties;
  var
    I, J: Integer;
  begin
    vFirstIterate := True;
    for J := 0 to AEntities.Count - 1 do
    begin
      if (AType <> nil) and (AType <> TObject(AEntities[J]).ClassType) then
        Continue;
      vEntity := TsgDXFEntity(AEntities[J]);
      vList := vEntity.GetProperties;
      vInd := 1;
      for I := 0 to vList.Count - 1 do
      begin
        vObj := TsgObject(vList[I]);
        if FProperties.IndexOf(vObj.ClassType) = -1 then
          Continue;
        if I <> 0 then
          AddRow;
        StringGridObject.Rows[vInd].Strings[0] := vObj.GetPropertyName;
        AddValue(vObj.GetStrValue, vObj, vEntity);
        if vObj is TsgFPoint then
        begin
          vPointClass := TsgFPointClass(vObj.ClassType);
          StringGridObject.Rows[vInd].Objects[0] := nil;
          AddRow;
          StringGridObject.Rows[vInd].Strings[0] := '     X';
          AddValue(FloatToStr(TsgFPoint(vObj).X), vPointClass.CreateOneX(TsgFPoint(vObj).X), vEntity);
          AddRow;
          StringGridObject.Rows[vInd].Strings[0] := '     Y';
          AddValue(FloatToStr(TsgFPoint(vObj).Y), vPointClass.CreateOneY(TsgFPoint(vObj).Y), vEntity);
          AddRow;
          StringGridObject.Rows[vInd].Strings[0] := '     Z';
          AddValue(FloatToStr(TsgFPoint(vObj).Z), vPointClass.CreateOneZ(TsgFPoint(vObj).Z), vEntity);
        end;
      end;
      vFirstIterate := False;
    end;
  end;

  procedure AddListObject(AListObj: TsgListObject);
  var
    I: Integer;
    vSubList: TList;
    vSpaces: string;
    vList: TList;
    vSubType: TsgListTypeProps;
  begin
    Inc(vStackCount);
    vFirstIterate := True;
    vSubList := AListObj.GetSubEntities;
    for I := 0 to vStackCount - 1 do
      vSpaces := vSpaces + '    ';
    if FEntities[0] <> AListObj then
    begin
      vList := AListObj.GetProperties;
      for I := 0 to vList.Count - 1 do
      begin
        vObj := TsgObject(vList[I]);
        AddRow;
        StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + vObj.GetPropertyName;
        AddValue(vSpaces + vObj.GetStrValue, vObj, AListObj);
        if vObj is TsgFPoint then
        begin
          vPointClass := TsgFPointClass(vObj.ClassType);
          StringGridObject.Rows[StringGridObject.RowCount - 1].Objects[0] := nil;
          AddRow;
          StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     X';
          AddValue(FloatToStr(TsgFPoint(vObj).X), vPointClass.CreateOneX(TsgFPoint(vObj).X), AListObj);
          AddRow;
          StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     Y';
          AddValue(FloatToStr(TsgFPoint(vObj).Y), vPointClass.CreateOneY(TsgFPoint(vObj).Y), AListObj);
          AddRow;
          StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     Z';
          AddValue(FloatToStr(TsgFPoint(vObj).Z), vPointClass.CreateOneZ(TsgFPoint(vObj).Z), AListObj);
        end;
      end;
    end;
    vSubType := AListObj.GetSubEntitiesType;
    if (vSubType = ltIndexed) and IndexesAsPluses then
      vSubType := ltPlus;
    case vSubType of
      ltAttribs:
        begin
          if (vSubList.Count > 0) and vFirstIterate then
          begin
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + 'Attributes';
          end;
          for I := 0 to vSubList.Count - 1 do
          begin
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     ' + TsgDXFAttrib(vSubList[I]).Tag;
            AddValue(vSpaces + TsgDXFAttrib(vSubList[I]).Value, TsgDXFAttrib(vSubList[I]), TsgDXFAttrib(vSubList[I]));
          end;
        end;
      ltIndexed:
        begin
          FVertexIndex.MaxValue := vSubList.Count - 1;
          vListObj := TsgListObject(vSubList[FVertexIndex.Value]);
          vList := vListObj.GetProperties;
          if (vList.Count > 0) and vFirstIterate then
          begin
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + 'Vertexes';
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     Count:';
            AddValue(IntToStr(vSubList.Count), nil, nil);
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     Index:';
            AddValue(IntToStr(FVertexIndex.Value), FVertexIndex, nil);
          end;
          for I := 0 to vList.Count - 1 do
          begin
            AddRow;
            vObj := TsgObject(vList[I]);
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '     ' + TsgObject(vList[I]).GetPropertyName;
            AddValue(vSpaces + TsgObject(vList[I]).GetStrValue, vObj, vListObj);
            if vObj is TsgFPoint then
            begin
              vPointClass := TsgFPointClass(vObj.ClassType);
              AddRow;
              StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '          X';
              AddValue(FloatToStr(TsgFPoint(vObj).X), vPointClass.CreateOneX(TsgFPoint(vObj).X), vListObj);
              AddRow;
              StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '          Y';
              AddValue(vSpaces + FloatToStr(TsgFPoint(vObj).Y), vPointClass.CreateOneY(TsgFPoint(vObj).Y), vListObj);
              if not (vEntity is TsgDXFLWPolyline) then //?? should be normal ierarhy in dxfconv!! this is a patch
              begin
                AddRow;
                StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces +  '          Z';
                AddValue(vSpaces + FloatToStr(TsgFPoint(vObj).Z), vPointClass.CreateOneZ(TsgFPoint(vObj).Z), vListObj);
              end;
            end;
          end;
        end;
      ltPlus:
        begin
          for I := 0 to vSubList.Count - 1 do
          begin
            AddRow;
            StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '+';
            AddValue(vSpaces + TsgListObject(vSubList[I]).GetStrValue, TsgListObject(vSubList[I]), TsgListObject(vSubList[I]));
            if FMinuses.IndexOf(TsgListObject(vSubList[I])) <> -1 then
            begin
             StringGridObject.Rows[StringGridObject.RowCount - 1].Strings[0] := vSpaces + '-';
             AddListObject(TsgListObject(vSubList[I]));
            end;
          end;
        end;
    end;
    Dec(vStackCount);
  end;

begin
  if (AEntities = nil) or (AEntities.Count = 0) then
  begin
    Clear;
    Exit;
  end;
  GlobalsgObjects.ClearFull;
  vTypes := TList.Create;
  vCounts := TList.Create;
  if AEntities <> FEntities then
  begin
    FEntities.Assign(AEntities);
    FVertexIndex.Value := 0;
  end;
  StringGridObject.RowCount := 2;
  FProperties.Clear;

  FindSameProperties;

  AddProperties;

  vStackCount := 0;
  if FEntities.Count = 1 then
    AddListObject(TsgListObject(FEntities[0]));
  if AType = nil then
  begin
    cbSelectedEntities.Clear;
    for I := 0 to vTypes.Count - 1 do
      cbSelectedEntities.AddItem(TsgObjectClass(vTypes[I]).GetPropertyName + ' (' + IntToStr(Integer(vCounts[I])) + ')',  vTypes[I]);
    if vTypes.Count > 1 then
      cbSelectedEntities.Items.Insert(0, 'All (' + IntToStr(FEntities.Count) + ')');
    cbSelectedEntities.ItemIndex := 0;
  end;
  vCounts.Free;
  vTypes.Free;
end;

procedure TfmEntityInspector.ComboBox1Change(Sender: TObject);
begin
  StringGridObject.Cells[StringGridObject.Col,
                    StringGridObject.Row] :=
    ComboBox1.Items[ComboBox1.ItemIndex];
  ChangeObjectParams(TsgObject(ComboBox1.Items.Objects[ComboBox1.ItemIndex]));
end;

procedure TfmEntityInspector.ComboBox1Exit(Sender: TObject);
begin
  ComboBox1.Visible := False;
end;

procedure TfmEntityInspector.StringGridObjectKeyPress(Sender: TObject;
  var Key: Char);
var
  vProp: TsgObject;
  vPos: Integer;
begin
  if StringGridObject.Rows[FRow].Objects[0] = nil then
    Key := #0;
  if Key = #13 then
  begin
    vPos := StringGridObject.TopRow;
    vProp := TsgObject(StringGridObject.Rows[FRow].Objects[0]);
    if vProp = nil then
      Exit;
    vProp.SetStrValue(StringGridObject.Rows[FRow].Strings[1]);
    ChangeObjectParams(vProp);
    StringGridObject.Row := FRow;
    StringGridObject.TopRow := vPos;
  end;
end;

procedure TfmEntityInspector.StringGridObjectMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 vColumn, vRow, vPos: Integer;
begin
  StringGridObject.MouseToCell(X, Y, vColumn, vRow);
  FRow := vRow;
  if vColumn > 0 then
    Exit;
  vPos := StringGridObject.TopRow;
  if Trim(StringGridObject.Cells[0, vRow]) = '+' then
  begin
    FMinuses.Add(StringGridObject.Rows[vRow].Objects[0]);
    RefreshGrid(FEntities, TsgObjectClass(nil));
  end
  else
    if Trim(StringGridObject.Cells[0, vRow]) = '-' then
    begin
      FMinuses.Remove(StringGridObject.Rows[vRow].Objects[0]);
      RefreshGrid(FEntities, TsgObjectClass(nil));
    end;
  StringGridObject.Row := vRow;
  StringGridObject.TopRow := vPos;
end;

procedure TfmEntityInspector.StringGridObjectMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  vInitW: Integer;
begin
//  SetFocus;
  if (StringGridObject.Cursor = crHSplit) and (ssLeft in Shift) then
  begin
    vInitW := StringGridObject.ColWidths[0];
    if (X > 50) and (vInitW - X + StringGridObject.ColWidths[1] > 50) then
    StringGridObject.ColWidths[0] := X;
    StringGridObject.ColWidths[1] := StringGridObject.ColWidths[1] + vInitW - X;
  end;
  if (Abs(X - StringGridObject.ColWidths[0]) < 5) then
  begin
    StringGridObject.Cursor := crHSplit;
  end
  else
    StringGridObject.Cursor := crDefault;
end;

procedure TfmEntityInspector.StringGridObjectSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var
  R: TRect;
  I: Integer;
  vObj: TsgObject;
  S: string;
begin
  FRow := ARow;
  S := Trim(StringGridObject.Cells[0, FRow]);

  if (S = '+') or (S = '-') then
    Exit;
  vObj := TsgObject(StringGridObject.Rows[ARow].Objects[0]);
  if (vObj <> nil) and ((vObj.ClassType = TsgDXFLayer) or (vObj.ClassType = TsgDXFLineType)) then
  begin
    ComboBox1.Clear;
    if vObj.ClassType = TsgDXFLayer then
      for I := 0 to CADImage.Converter.Counts[csLayers] - 1 do
        ComboBox1.AddItem(CADImage.Converter.Layers[I].Name, CADImage.Converter.Layers[I]);
    if vObj.ClassType = TsgDXFLineType then
      for I := 0 to CADImage.Converter.Counts[csLTypes] - 1 do
        ComboBox1.AddItem(CADImage.Converter.LTypes[I].Name, CADImage.Converter.LTypes[I]);
   {   combobox   }
    R := StringGridObject.CellRect(ACol, ARow);
    R.Left := R.Left + StringGridObject.Left;
    R.Right := R.Right + StringGridObject.Left;
    R.Top := R.Top + StringGridObject.Top;
    R.Bottom := R.Bottom + StringGridObject.Top;
    ComboBox1.Left := R.Left + 1;
    ComboBox1.Top := R.Top + 1;
    ComboBox1.Width := (R.Right + 1) - R.Left;
    ComboBox1.Height := (R.Bottom + 1) - R.Top;
   { combobox}
    ComboBox1.DroppedDown := True;
    ComboBox1.ItemIndex := ComboBox1.Items.IndexOfObject(vObj);
    ComboBox1.Visible := True;
    ComboBox1.SetFocus;
  end;
  CanSelect := True;
end;

procedure TfmEntityInspector.StringGridObjectDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
//var
//  vColor: TColor;
begin
  StringGridObject.Canvas.Font.Style := [fsBold];
{  if ARow = StringGridObject.RowCount - 1 then
    Exit;
  S := Trim(StringGridObject.Cells[0, ARow + 1]);
  if not ((S = '+') or (S = '-')) then
    Exit;
  vPenWidth := StringGridObject.Canvas.Pen.Width;
  vColor := StringGridObject.Canvas.Pen.Color;
  StringGridObject.Canvas.Pen.Color := clBlack;
  StringGridObject.Canvas.MoveTo(Rect.Left, Rect.Bottom);
  StringGridObject.Canvas.LineTo(Rect.Right, Rect.Bottom);
  StringGridObject.Canvas.Pen.Width := vPenWidth;
  StringGridObject.Canvas.Pen.Color := vColor;}
end;

procedure TfmEntityInspector.StringGridObjectExit(Sender: TObject);
begin
 if ChangeCellText then
   StringGridObject.Cells[FColumn,FRow] := FPrevCellText;
end;

procedure TfmEntityInspector.cbSelectedEntitiesChange(Sender: TObject);
begin
  RefreshGrid(FEntities, TsgObjectClass(cbSelectedEntities.Items.Objects[cbSelectedEntities.ItemIndex]));
end;

procedure TfmEntityInspector.ChangeObjectParams(AProperty: TsgObject; AEntity: TsgDXFEntity);
var
  I, vPos, vRow: Integer;
  vType: TsgObjectClass;
  vEntity: TsgListObject;
begin
  vPos := StringGridObject.TopRow;
  vRow := StringGridObject.Row;
  vType := TsgObjectClass(cbSelectedEntities.Items.Objects[cbSelectedEntities.ItemIndex]);
  if FEntities.Count > 1  then
    for I := 0 to FEntities.Count - 1 do
    begin
      if (vType <> nil) and (TsgDXFEntity(FEntities[I]).ClassType <> vType) then
        Continue;
      TsgDXFEntity(FEntities[I]).SetProperty(AProperty);
      CADImage.Converter.Loads(TsgDXFEntity(FEntities[I]));
    end
  else //if there is one entity
  begin
    vEntity := TsgListObject(StringGridObject.Rows[FRow].Objects[1]);
    if vEntity <> nil then
    begin
      vEntity.SetProperty(AProperty);
      if not (vEntity  is TsgDXFConverter) then
        CADImage.Converter.Loads(TsgDXFEntity(vEntity));
    end;
    if not (TObject(FEntities[0]) is TsgDXFConverter) then
      CADImage.Converter.Loads(TsgDXFEntity(FEntities[0]));
  end;
  FOnSetValue;
  RefreshGrid(FEntities, vType);
  StringGridObject.Row := vRow - 1;
  StringGridObject.TopRow := vPos;
end;

procedure TfmEntityInspector.Load(AImage: TsgDrawingNavigator);
var
  I: Integer;
begin
  Clear;
  FDNavigator := AImage;
  for I := 0 to CADImage.Converter.Counts[csEntities] - 1 do
    AddEntity(CADImage.Converter.Sections[csEntities].Entities[I], False);
end;

procedure TfmEntityInspector.Clear;
begin
  StringGridObject.RowCount := 1;
  cbSelectedEntities.Clear;
  if (FDNavigator <> nil) and (CADImage <> nil) then
  begin
    CADImage.Converter.Loads(FEntitySample);
    fmEntityInspector.RefreshGrid(FEntitySample);

    fmEntityInspector.RefreshGrid(CADImage.Converter);
    cbSelectedEntities.ItemIndex := 0;
  end;
end;

procedure TfmEntityInspector.AddEntity(AEnt: TsgDXFEntity; ARefresh: Boolean);
begin
  if ARefresh then
    RefreshGrid(AEnt);
end;

procedure TfmEntityInspector.SaveDrawing1Click(Sender: TObject);
var
  vXML: TsgCADtoXML;
begin
  if CADImage <> nil then
  begin
    if not SaveDialog1.Execute then
      Exit;
    vXML := TsgCADtoXML.Create;
    try
      vXML.ExportEntity(CADImage.Converter);
      vXML.XML.SaveToFile(SaveDialog1.FileName);
    finally
      vXML.Free;
      ShellExecute(Handle, 'open', PChar(SaveDialog1.FileName), '', '', SW_SHOWNORMAL) ;
    end;
  end;
end;

procedure TfmEntityInspector.SavetoXML1Click(Sender: TObject);
var
  vXML: TsgCADtoXML;
begin
  if FRow = -1 then
    FRow := 2;
  if (FEntities.Count > 0) and (StringGridObject.Rows[FRow].Objects[1] <> nil) then
  begin
    if not SaveDialog1.Execute then
      Exit;
    vXML := TsgCADtoXML.Create;
    try
      vXML.ExportEntity(TsgListObject(StringGridObject.Rows[FRow].Objects[1]));
      vXML.XML.SaveToFile(SaveDialog1.FileName);
    finally
      vXML.Free;
      ShellExecute(Handle, 'open', PChar(SaveDialog1.FileName), '', '', SW_SHOWNORMAL) ;
    end;
  end;
end;

procedure TfmEntityInspector.SetChangeCellText(const Value: Boolean);
begin
  FChangeCellText := Value;
end;

procedure TfmEntityInspector.SetIndexesAsPluses(const Value: Boolean);
begin
  FIndexesAsPluses := Value;
  RefreshGrid(FEntities, TsgObjectClass(nil));
end;

procedure TfmEntityInspector.SetOnSetValue(const Value: TsgOnSetValue);
begin
  FOnSetValue := Value;
end;

procedure TfmEntityInspector.UpdateEntity(AEntity: TsgDXFEntity);
var
  vInvRect: TRect;
begin
  if AEntity <> nil then
  begin
    vInvRect := FDNavigator.PictureRect;
    InvalidateRect(FDNavigator.Parent.Handle, @vInvRect, False);
  end;
end;

function TfmEntityInspector.GetCADImage: TsgCADImage;
begin
  Result := TsgCADImage(FDNavigator.Picture.Graphic);
end;

procedure TfmEntityInspector.FormShow(Sender: TObject);
begin
  // To stay main form foreground

end;

procedure TfmEntityInspector.SpeedButtonStayOnTopMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if SpeedButtonStayOnTop.Down then
    SetWindowPos(Handle, HWND_NOTOPMOST, Left, Top, Width, Height, SWP_SHOWWINDOW)
  else
    SetWindowPos(Handle, HWND_TOPMOST, Left, Top, Width, Height, SWP_SHOWWINDOW);
end;

procedure TfmEntityInspector.pnlComboResize(Sender: TObject);
begin
  SpeedButtonStayOnTop.Left := pnlCombo.Width - SpeedButtonStayOnTop.Width;
  cbSelectedEntities.Width := SpeedButtonStayOnTop.Left;
end;

procedure TfmEntityInspector.RefreshGrid(AEntity: TsgListObject);
var
  vList: TList;
begin
  vList := TList.Create;
  vList.Add(AEntity);
  RefreshGrid(vList, TsgObjectClass(nil));
  vList.Free;
end;

end.
